/* Main Simulation File */

#if defined(__cplusplus)
extern "C" {
#endif

#include "WaterTank_Control_model.h"
#include "simulation/solver/events.h"


/* dummy VARINFO and FILEINFO */
const FILE_INFO dummyFILE_INFO = omc_dummyFileInfo;
const VAR_INFO dummyVAR_INFO = omc_dummyVarInfo;

int WaterTank_Control_input_function(DATA *data, threadData_t *threadData)
{
  TRACE_PUSH

  data->localData[0]->realVars[1] /* level variable */ = data->simulationInfo->inputVars[0];
  
  TRACE_POP
  return 0;
}

int WaterTank_Control_input_function_init(DATA *data, threadData_t *threadData)
{
  TRACE_PUSH

  data->simulationInfo->inputVars[0] = data->modelData->realVarsData[1].attribute.start;
  
  TRACE_POP
  return 0;
}

int WaterTank_Control_input_function_updateStartValues(DATA *data, threadData_t *threadData)
{
  TRACE_PUSH

  data->modelData->realVarsData[1].attribute.start = data->simulationInfo->inputVars[0];
  
  TRACE_POP
  return 0;
}

int WaterTank_Control_inputNames(DATA *data, char ** names){
  TRACE_PUSH

  names[0] = (char *) data->modelData->realVarsData[1].info.name;
  
  TRACE_POP
  return 0;
}

int WaterTank_Control_data_function(DATA *data, threadData_t *threadData)
{
  TRACE_PUSH

  TRACE_POP
  return 0;
}

int WaterTank_Control_output_function(DATA *data, threadData_t *threadData)
{
  TRACE_PUSH

  data->simulationInfo->outputVars[0] = data->localData[0]->realVars[2] /* valve variable */;
  
  TRACE_POP
  return 0;
}

int WaterTank_Control_setc_function(DATA *data, threadData_t *threadData)
{
  TRACE_PUSH

  
  TRACE_POP
  return 0;
}


/*
equation index: 7
type: SIMPLE_ASSIGN
$outputAlias_valve = if level >= maxlevel then 1.0 else if level < minlevel then 0.0 else pre($outputAlias_valve)
*/
void WaterTank_Control_eqFunction_7(DATA *data, threadData_t *threadData)
{
  TRACE_PUSH
  const int equationIndexes[2] = {1,7};
  modelica_boolean tmp0;
  modelica_boolean tmp1;
  modelica_boolean tmp2;
  modelica_real tmp3;
  RELATIONHYSTERESIS(tmp0, data->localData[0]->realVars[1] /* level variable */, data->simulationInfo->realParameter[0] /* maxlevel PARAM */, 0, GreaterEq);
  tmp2 = (modelica_boolean)tmp0;
  if(tmp2)
  {
    tmp3 = 1.0;
  }
  else
  {
    RELATIONHYSTERESIS(tmp1, data->localData[0]->realVars[1] /* level variable */, data->simulationInfo->realParameter[1] /* minlevel PARAM */, 1, Less);
    tmp3 = (tmp1?0.0:data->simulationInfo->realVarsPre[0] /* $outputAlias_valve variable */);
  }
  data->localData[0]->realVars[0] /* $outputAlias_valve variable */ = tmp3;
  TRACE_POP
}
/*
equation index: 8
type: SIMPLE_ASSIGN
valve = $outputAlias_valve
*/
void WaterTank_Control_eqFunction_8(DATA *data, threadData_t *threadData)
{
  TRACE_PUSH
  const int equationIndexes[2] = {1,8};
  data->localData[0]->realVars[2] /* valve variable */ = data->localData[0]->realVars[0] /* $outputAlias_valve variable */;
  TRACE_POP
}

OMC_DISABLE_OPT
int WaterTank_Control_functionDAE(DATA *data, threadData_t *threadData)
{
  TRACE_PUSH
  int equationIndexes[1] = {0};
#if !defined(OMC_MINIMAL_RUNTIME)
  if (measure_time_flag) rt_tick(SIM_TIMER_DAE);
#endif

  data->simulationInfo->needToIterate = 0;
  data->simulationInfo->discreteCall = 1;
  WaterTank_Control_functionLocalKnownVars(data, threadData);
  WaterTank_Control_eqFunction_7(data, threadData);

  WaterTank_Control_eqFunction_8(data, threadData);
  data->simulationInfo->discreteCall = 0;
  
#if !defined(OMC_MINIMAL_RUNTIME)
  if (measure_time_flag) rt_accumulate(SIM_TIMER_DAE);
#endif
  TRACE_POP
  return 0;
}


int WaterTank_Control_functionLocalKnownVars(DATA *data, threadData_t *threadData)
{
  TRACE_PUSH

  
  TRACE_POP
  return 0;
}


int WaterTank_Control_functionODE(DATA *data, threadData_t *threadData)
{
  TRACE_PUSH
#if !defined(OMC_MINIMAL_RUNTIME)
  if (measure_time_flag) rt_tick(SIM_TIMER_FUNCTION_ODE);
#endif

  
  data->simulationInfo->callStatistics.functionODE++;
  
  WaterTank_Control_functionLocalKnownVars(data, threadData);
  /* no ODE systems */

#if !defined(OMC_MINIMAL_RUNTIME)
  if (measure_time_flag) rt_accumulate(SIM_TIMER_FUNCTION_ODE);
#endif

  TRACE_POP
  return 0;
}

/* forward the main in the simulation runtime */
extern int _main_SimulationRuntime(int argc, char**argv, DATA *data, threadData_t *threadData);

#include "WaterTank_Control_12jac.h"
#include "WaterTank_Control_13opt.h"

struct OpenModelicaGeneratedFunctionCallbacks WaterTank_Control_callback = {
   NULL,
   NULL,
   NULL,
   WaterTank_Control_callExternalObjectDestructors,
   NULL,
   WaterTank_Control_initialLinearSystem,
   NULL,
   #if !defined(OMC_NO_STATESELECTION)
   WaterTank_Control_initializeStateSets,
   #else
   NULL,
   #endif
   WaterTank_Control_initializeDAEmodeData,
   WaterTank_Control_functionODE,
   WaterTank_Control_functionAlgebraics,
   WaterTank_Control_functionDAE,
   WaterTank_Control_functionLocalKnownVars,
   WaterTank_Control_input_function,
   WaterTank_Control_input_function_init,
   WaterTank_Control_input_function_updateStartValues,
   WaterTank_Control_data_function,
   WaterTank_Control_output_function,
   WaterTank_Control_setc_function,
   WaterTank_Control_function_storeDelayed,
   WaterTank_Control_updateBoundVariableAttributes,
   WaterTank_Control_functionInitialEquations,
   1, /* useHomotopy - 0: local homotopy (equidistant lambda), 1: global homotopy (equidistant lambda), 2: new global homotopy approach (adaptive lambda), 3: new local homotopy approach (adaptive lambda)*/
   WaterTank_Control_functionInitialEquations_lambda0,
   WaterTank_Control_functionRemovedInitialEquations,
   WaterTank_Control_updateBoundParameters,
   WaterTank_Control_checkForAsserts,
   WaterTank_Control_function_ZeroCrossingsEquations,
   WaterTank_Control_function_ZeroCrossings,
   WaterTank_Control_function_updateRelations,
   WaterTank_Control_zeroCrossingDescription,
   WaterTank_Control_relationDescription,
   WaterTank_Control_function_initSample,
   WaterTank_Control_INDEX_JAC_A,
   WaterTank_Control_INDEX_JAC_B,
   WaterTank_Control_INDEX_JAC_C,
   WaterTank_Control_INDEX_JAC_D,
   WaterTank_Control_INDEX_JAC_F,
   WaterTank_Control_initialAnalyticJacobianA,
   WaterTank_Control_initialAnalyticJacobianB,
   WaterTank_Control_initialAnalyticJacobianC,
   WaterTank_Control_initialAnalyticJacobianD,
   WaterTank_Control_initialAnalyticJacobianF,
   WaterTank_Control_functionJacA_column,
   WaterTank_Control_functionJacB_column,
   WaterTank_Control_functionJacC_column,
   WaterTank_Control_functionJacD_column,
   WaterTank_Control_functionJacF_column,
   WaterTank_Control_linear_model_frame,
   WaterTank_Control_linear_model_datarecovery_frame,
   WaterTank_Control_mayer,
   WaterTank_Control_lagrange,
   WaterTank_Control_pickUpBoundsForInputsInOptimization,
   WaterTank_Control_setInputData,
   WaterTank_Control_getTimeGrid,
   WaterTank_Control_symbolicInlineSystem,
   WaterTank_Control_function_initSynchronous,
   WaterTank_Control_function_updateSynchronous,
   WaterTank_Control_function_equationsSynchronous,
   WaterTank_Control_inputNames,
   WaterTank_Control_read_input_fmu,
   NULL,
   NULL,
   -1

};

#define _OMC_LIT_RESOURCE_0_name_data "Complex"
#define _OMC_LIT_RESOURCE_0_dir_data "/var/lib/jenkins3/ws/OpenModelica_maintenance_v1.14/build/lib/omlibrary"
static const MMC_DEFSTRINGLIT(_OMC_LIT_RESOURCE_0_name,7,_OMC_LIT_RESOURCE_0_name_data);
static const MMC_DEFSTRINGLIT(_OMC_LIT_RESOURCE_0_dir,71,_OMC_LIT_RESOURCE_0_dir_data);

#define _OMC_LIT_RESOURCE_1_name_data "FmuExportCrossCompile"
#define _OMC_LIT_RESOURCE_1_dir_data "."
static const MMC_DEFSTRINGLIT(_OMC_LIT_RESOURCE_1_name,21,_OMC_LIT_RESOURCE_1_name_data);
static const MMC_DEFSTRINGLIT(_OMC_LIT_RESOURCE_1_dir,1,_OMC_LIT_RESOURCE_1_dir_data);

#define _OMC_LIT_RESOURCE_2_name_data "Modelica"
#define _OMC_LIT_RESOURCE_2_dir_data "/var/lib/jenkins3/ws/OpenModelica_maintenance_v1.14/build/lib/omlibrary/Modelica 3.2.3"
static const MMC_DEFSTRINGLIT(_OMC_LIT_RESOURCE_2_name,8,_OMC_LIT_RESOURCE_2_name_data);
static const MMC_DEFSTRINGLIT(_OMC_LIT_RESOURCE_2_dir,86,_OMC_LIT_RESOURCE_2_dir_data);

#define _OMC_LIT_RESOURCE_3_name_data "ModelicaServices"
#define _OMC_LIT_RESOURCE_3_dir_data "/var/lib/jenkins3/ws/OpenModelica_maintenance_v1.14/build/lib/omlibrary/ModelicaServices 3.2.3"
static const MMC_DEFSTRINGLIT(_OMC_LIT_RESOURCE_3_name,16,_OMC_LIT_RESOURCE_3_name_data);
static const MMC_DEFSTRINGLIT(_OMC_LIT_RESOURCE_3_dir,94,_OMC_LIT_RESOURCE_3_dir_data);

#define _OMC_LIT_RESOURCE_4_name_data "RoomHeating_OM"
#define _OMC_LIT_RESOURCE_4_dir_data "/var/lib/jenkins3/ws/OpenModelica_maintenance_v1.14/testsuite/special/FmuExportCrossCompile"
static const MMC_DEFSTRINGLIT(_OMC_LIT_RESOURCE_4_name,14,_OMC_LIT_RESOURCE_4_name_data);
static const MMC_DEFSTRINGLIT(_OMC_LIT_RESOURCE_4_dir,91,_OMC_LIT_RESOURCE_4_dir_data);

#define _OMC_LIT_RESOURCE_5_name_data "WaterTank"
#define _OMC_LIT_RESOURCE_5_dir_data "/var/lib/jenkins3/ws/OpenModelica_maintenance_v1.14/testsuite/special/FmuExportCrossCompile"
static const MMC_DEFSTRINGLIT(_OMC_LIT_RESOURCE_5_name,9,_OMC_LIT_RESOURCE_5_name_data);
static const MMC_DEFSTRINGLIT(_OMC_LIT_RESOURCE_5_dir,91,_OMC_LIT_RESOURCE_5_dir_data);

static const MMC_DEFSTRUCTLIT(_OMC_LIT_RESOURCES,12,MMC_ARRAY_TAG) {MMC_REFSTRINGLIT(_OMC_LIT_RESOURCE_0_name), MMC_REFSTRINGLIT(_OMC_LIT_RESOURCE_0_dir), MMC_REFSTRINGLIT(_OMC_LIT_RESOURCE_1_name), MMC_REFSTRINGLIT(_OMC_LIT_RESOURCE_1_dir), MMC_REFSTRINGLIT(_OMC_LIT_RESOURCE_2_name), MMC_REFSTRINGLIT(_OMC_LIT_RESOURCE_2_dir), MMC_REFSTRINGLIT(_OMC_LIT_RESOURCE_3_name), MMC_REFSTRINGLIT(_OMC_LIT_RESOURCE_3_dir), MMC_REFSTRINGLIT(_OMC_LIT_RESOURCE_4_name), MMC_REFSTRINGLIT(_OMC_LIT_RESOURCE_4_dir), MMC_REFSTRINGLIT(_OMC_LIT_RESOURCE_5_name), MMC_REFSTRINGLIT(_OMC_LIT_RESOURCE_5_dir)}};
void WaterTank_Control_setupDataStruc(DATA *data, threadData_t *threadData)
{
  assertStreamPrint(threadData,0!=data, "Error while initialize Data");
  threadData->localRoots[LOCAL_ROOT_SIMULATION_DATA] = data;
  data->callback = &WaterTank_Control_callback;
  OpenModelica_updateUriMapping(threadData, MMC_REFSTRUCTLIT(_OMC_LIT_RESOURCES));
  data->modelData->modelName = "WaterTank.Control";
  data->modelData->modelFilePrefix = "WaterTank_Control";
  data->modelData->resultFileName = NULL;
  data->modelData->modelDir = "/var/lib/jenkins3/ws/OpenModelica_maintenance_v1.14/testsuite/special/FmuExportCrossCompile";
  data->modelData->modelGUID = "{5da4c67c-5306-41eb-a7cc-590f252e782c}";
  data->modelData->initXMLData = NULL;
  data->modelData->modelDataXml.infoXMLData = NULL;
  
  data->modelData->nStates = 0;
  data->modelData->nVariablesReal = 3;
  data->modelData->nDiscreteReal = 0;
  data->modelData->nVariablesInteger = 0;
  data->modelData->nVariablesBoolean = 0;
  data->modelData->nVariablesString = 0;
  data->modelData->nParametersReal = 2;
  data->modelData->nParametersInteger = 0;
  data->modelData->nParametersBoolean = 0;
  data->modelData->nParametersString = 0;
  data->modelData->nInputVars = 1;
  data->modelData->nOutputVars = 1;
  
  data->modelData->nAliasReal = 0;
  data->modelData->nAliasInteger = 0;
  data->modelData->nAliasBoolean = 0;
  data->modelData->nAliasString = 0;
  
  data->modelData->nZeroCrossings = 2;
  data->modelData->nSamples = 0;
  data->modelData->nRelations = 2;
  data->modelData->nMathEvents = 0;
  data->modelData->nExtObjs = 0;
  
  GC_asprintf(&data->modelData->modelDataXml.fileName, "%s/WaterTank_Control_info.json", data->modelData->resourcesDir);
  data->modelData->modelDataXml.modelInfoXmlLength = 0;
  data->modelData->modelDataXml.nFunctions = 0;
  data->modelData->modelDataXml.nProfileBlocks = 0;
  data->modelData->modelDataXml.nEquations = 9;
  data->modelData->nMixedSystems = 0;
  data->modelData->nLinearSystems = 1;
  data->modelData->nNonLinearSystems = 0;
  data->modelData->nStateSets = 0;
  data->modelData->nJacobians = 6;
  data->modelData->nOptimizeConstraints = 0;
  data->modelData->nOptimizeFinalConstraints = 0;
  
  data->modelData->nDelayExpressions = 0;
  
  data->modelData->nClocks = 0;
  data->modelData->nSubClocks = 0;
  
  data->modelData->nSensitivityVars = 0;
  data->modelData->nSensitivityParamVars = 0;
  data->modelData->nSetcVars = 0;
  data->modelData->ndataReconVars = 0;
}

static int rml_execution_failed()
{
  fflush(NULL);
  fprintf(stderr, "Execution failed!\n");
  fflush(NULL);
  return 1;
}

